"""Tests of waelet tool."""
import pyspedas
from pyspedas import get_data, cdf_to_tplot, tplot, data_exists, tplot_rename, tplot_copy, ylim, zlim, options
import unittest
from numpy.testing import assert_array_almost_equal_nulp, assert_array_max_ulp, assert_allclose
import numpy as np
from pyspedas.analysis.wavelet import wavelet
import logging

global_display = True

class TwaveletDataValidation(unittest.TestCase):
    """ Compares wavelet results between Python and IDL """

    @classmethod
    def setUpClass(cls):
        """
        IDL Data has to be downloaded to perform these tests
        The SPEDAS script that creates the file: general/tools/tplot/wvlt/wavelet_python_validate.pro
        """
        from pyspedas.utilities.download import download
        from pyspedas.projects.themis.config import CONFIG

        # Testing tolerance
        cls.tol = 1e-10

        # Download tplot files
        remote_server = 'https://github.com/spedas/test_data/raw/refs/heads/main/'
        remote_name = 'analysis_tools/wavelet_test.tplot'
        datafile = download(remote_file=remote_name,
                            remote_path=remote_server,
                            local_path=CONFIG['local_data_dir'],
                            no_download=False)
        if not datafile:
            # Skip tests
            raise unittest.SkipTest("Cannot download data validation file")
        # Load validation variables from the test file
        pyspedas.del_data('*')
        filename = datafile[0]
        # pyspedas.cdf_to_tplot(filename)
        pyspedas.tplot_restore(filename)
        pyspedas.tplot_names()

        # Test 1: simple sine waves
        cls.sin_wav = pyspedas.get_data('sin_wav')
        cls.sin_wav_wv_pow = pyspedas.get_data('sin_wav_wv_pow')

        # Test 2: Vassilis homework example using THEMIS FGM data
        cls.tha_fgs_fac_bp_x = pyspedas.get_data('tha_fgs_fac_bp_x')
        cls.tha_fgs_fac_bp_y = pyspedas.get_data('tha_fgs_fac_bp_y')
        cls.tha_fgs_fac_bp_z = pyspedas.get_data('tha_fgs_fac_bp_z')
        cls.tha_fgs_fac_bp_x_wv_pow = pyspedas.get_data('tha_fgs_fac_bp_x_wv_pow')
        cls.tha_fgs_fac_bp_y_wv_pow = pyspedas.get_data('tha_fgs_fac_bp_y_wv_pow')
        cls.tha_fgs_fac_bp_z_wv_pow = pyspedas.get_data('tha_fgs_fac_bp_z_wv_pow')

        tplot_rename('sin_wav_wv_pow', 'idl_sin_wav_wv_pow')

        zlim('tha_fgs_fac_bp_?_wv_pow', 1.0e-1,1.0e2)
        ylim('tha_fgs_fac_bp_?_wv_pow', 1.0e-3,4.1e-2)
        options('tha_fgs_fac_bp_?_wv_pow', 'zlog', True)
        options('tha_fgs_fac_bp_?_wv_pow', 'ylog', False)

        tplot_rename('tha_fgs_fac_bp_x_wv_pow', 'idl_fgs_fac_bp_x_wv_pow')
        tplot_rename('tha_fgs_fac_bp_y_wv_pow', 'idl_fgs_fac_bp_y_wv_pow')
        tplot_rename('tha_fgs_fac_bp_z_wv_pow', 'idl_fgs_fac_bp_z_wv_pow')


    def setUp(self):
        """ We need to clean tplot variables before each run"""
        # pyspedas.del_data('*')

    def test_sin_wav(self):
        # The default is 'morl' rather than 'cmorl0.5-1.0, but it doesn't appear to make that much difference.
        # cmorl0.5-1.0 is what Eric used in the wiki example.
        wavelet('sin_wav',wavename='cmorl0.5-1.0')
        pvar='sin_wav_pow'
        pyspedas.options(pvar, 'colormap', 'jet')
        pyspedas.ylim(pvar, 0.001, 0.1)
        pyspedas.options(pvar, 'ylog', True)
        pyspedas.options(pvar, 'ytitle', pvar)
        d=get_data('sin_wav_pow')
        print(f"Python frequencies: bin count {d.y.shape[1]}, min {np.min(d.v)}, max: {np.max(d.v)}" )
        print(f"Python power: min {np.min(d.y)}, max {np.max(d.y)}")
        d_idl = get_data('idl_sin_wav_wv_pow')
        print(f"IDL frequencies: bin count {d_idl.y.shape[1]}, min {np.min(d_idl.v)}, max: {np.max(d_idl.v)}" )
        print(f"IDL power: min {np.min(d_idl.y)}, max {np.max(d_idl.y)}")
        tplot(['sin_wav', 'sin_wav_pow', 'idl_sin_wav_wv_pow'])

    def test_themis_fgm_wavelet(self):
        pyspedas.wavelet('tha_fgs_fac_bp_x')
        pyspedas.wavelet('tha_fgs_fac_bp_y')
        pyspedas.wavelet('tha_fgs_fac_bp_z')

        zlim('tha_fgs_fac_bp_?_pow', 1.0e-1,1.0e2)
        ylim('tha_fgs_fac_bp_?_pow', 1.0e-3,4.1e-2)
        options('tha_fgs_fac_bp_?_pow', 'zlog', True)
        options('tha_fgs_fac_bp_?_pow', 'ylog', False)
        d=get_data('tha_fgs_fac_bp_x_pow')
        print(f"Python frequencies: bin count {d.y.shape[1]}, min {np.min(d.v)}, max: {np.max(d.v)}" )
        print(f"Python power: min {np.min(d.y)}, max {np.max(d.y)}")
        d_idl=get_data('idl_fgs_fac_bp_x_wv_pow')
        print(f"IDL frequencies: bin count {d_idl.y.shape[1]}, min {np.min(d_idl.v)}, max: {np.max(d_idl.v)}" )
        print(f"IDL power: min {np.min(d_idl.y)}, max {np.max(d_idl.y)}")

        tplot(['tha_fgs_fac_bp_x_pow', 'idl_fgs_fac_bp_x_wv_pow'])

if __name__ == '__main__':
    unittest.main()



import unittest
import pyspedas
from viresclient import SwarmRequest
from pytplot import data_exists, del_data

class LoadTestCases(unittest.TestCase):
    def test_get_collections(self):
        from pyspedas.vires.config import CONFIG
        self.assertNotEqual(CONFIG['access_token'],'')
        collections = SwarmRequest(url="https://vires.services/ows",token=CONFIG['access_token']).available_collections(details=False)
        self.assertTrue('SW_OPER_MAGA_LR_1B' in collections['MAG'])
        self.assertTrue('SW_OPER_MODC_SC_1B' in collections['MOD_SC'])

    def test_load_mag_data(self):
        vires_vars = pyspedas.vires.load(trange=['2014-01-01T00:00', '2014-01-01T01:00'],
                                       collection="SW_OPER_MAGA_LR_1B",
                                       measurements=["F", "B_NEC"],
                                       models=["CHAOS-Core"],
                                       sampling_step="PT10S",
                                       auxiliaries=["QDLat", "QDLon"])
        self.assertTrue('Longitude' in vires_vars)
        self.assertTrue('Latitude' in vires_vars)
        self.assertTrue('B_NEC' in vires_vars)
        self.assertTrue(data_exists('Longitude'))
        self.assertTrue(data_exists('Latitude'))
        self.assertTrue(data_exists('B_NEC'))

    def test_load_no_trange(self):
        del_data('*')
        vires_vars = pyspedas.vires.load(collection="SW_OPER_MAGA_LR_1B",
                                       measurements=["F", "B_NEC"],
                                       models=["CHAOS-Core"],
                                       sampling_step="PT10S",
                                       auxiliaries=["QDLat", "QDLon"])
        self.assertTrue(vires_vars is None)
        self.assertFalse(data_exists('Longitude'))
        self.assertFalse(data_exists('Latitude'))
        self.assertFalse(data_exists('B_NEC'))

if __name__ == '__main__':
    unittest.main()



import unittest
from pyspedas import hapi
from pytplot import data_exists, del_data
from pyspedas.hapi_tools.replace_fillvals import replace_fillvals
import numpy as np
from numpy.testing import assert_array_equal


class HAPITests(unittest.TestCase):
    def test_replace_fillvals(self):
        da1_dbl = np.array([1.0, 2.0, 3.0, 4.0])
        fv1_dbl = 3.0
        replace_fillvals(da1_dbl, fv1_dbl, 'da1_dbl_scalarfill', 'double')
        assert_array_equal(da1_dbl, [1.0, 2.0, np.nan, 4.0])
        da2_dbl = np.array([[1.0, 2.0], [3.0, 4.0], [5.0, 6.0],[7.0, 2.0]])
        fv2_dbl = [1.0, 2.0]
        replace_fillvals(da2_dbl, fv2_dbl, 'da2_dbl_arrayfill', 'double')
        assert_array_equal(da2_dbl, np.array([[np.nan, np.nan], [3.0, 4.0], [5.0, 6.0],[7.0, np.nan]]))
        # Repeat after replacement, should be nothing to replace, and output array same as before
        fv2_dbl = [1.0, 2.0]
        replace_fillvals(da2_dbl, fv2_dbl, 'da2_dbl_arrayfill','double')
        assert_array_equal(da2_dbl, np.array([[np.nan, np.nan], [3.0, 4.0], [5.0, 6.0],[7.0, np.nan]]))
        da2_int = np.array([[0,1], [2,3], [4,5], [6,2]])
        da2_fvint = 2
        replace_fillvals(da2_int, da2_fvint, 'da2_int_scalarfill','integer')
        assert_array_equal(da2_int, np.array([[0,1],[0,3],[4,5],[6,0]]))


    def test_print_servers(self):
        with self.assertLogs(level='ERROR') as log: 
            hapi(trange=['2003-10-20', '2003-11-30'])
            self.assertIn("No server specified; example servers include:", log.output[0])

    def test_return_catalog(self):
        id_list = hapi(server='https://cdaweb.gsfc.nasa.gov/hapi', catalog=True, quiet=True)
        self.assertTrue('MMS1_EDI_BRST_L2_EFIELD' in id_list)

    def test_dataset_not_specified(self):
        # dataset not specified
        with self.assertLogs(level='ERROR') as log:
            h_vars = hapi(trange=['2003-10-20', '2003-11-30'],
                        server='https://cdaweb.gsfc.nasa.gov/hapi')
            self.assertIn("Error, no dataset specified; please see the catalog for a list of available data sets.", log.output[0])

    def test_trange_not_specified(self):
        # trange not specified
        with self.assertLogs(level='ERROR') as log:
            h_vars = hapi(dataset='OMNI_HRO2_1MIN',
                        server='https://cdaweb.gsfc.nasa.gov/hapi')
            self.assertIn("Error, no trange specified", log.output[0])

    def test_cdaweb_mms_spec(self):
        h_vars = hapi(trange=['2019-10-16', '2019-10-17'],
                      server='https://cdaweb.gsfc.nasa.gov/hapi',
                      dataset='MMS4_EDP_SRVY_L2_HFESP')

    def test_cdaweb_omni(self):
        del_data()
        h_vars = hapi(trange=['2003-10-20', '2003-11-30'],
                      server='https://cdaweb.gsfc.nasa.gov/hapi',
                      dataset='OMNI_HRO2_1MIN')
        self.assertTrue(data_exists('BX_GSE'))
        self.assertTrue(data_exists('BY_GSE'))
        self.assertTrue(data_exists('BZ_GSE'))

    def test_string_time(self):
        del_data()
        server = "https://supermag.jhuapl.edu/hapi"
        dataset = "ttb/PT1M/baseline_all"
        start = "2020-05-10T00:00Z"
        stop = "2020-05-14T00:00Z"
        parameters = ''
        param_list = hapi(trange=[start,stop], server=server, dataset=dataset, parameters=parameters)
        self.assertTrue('mlt' in param_list)
        self.assertTrue(data_exists('mlt'))

if __name__ == '__main__':
    unittest.main()




import unittest
import pyspedas
from pytplot import data_exists, tplot, get_data
import numpy as np

# Set this to False for Github CI testing, True to show the plots for interactive use
global_display=False

class MagNullTestCases(unittest.TestCase):


    def test_null_classification_x(self):
        # Test direct detection of 2-D type X nulls. (This will seldom or never occur in practice,
        # but see below for some degenerate cases that do appear in real data.
        l1 = complex(0.0,0.0)
        l2 = complex(-0.5, 0.0)
        l3 = complex(0.5,0.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,1)

    def test_null_classification_o(self):
        # Test direct detection of 2-D type O nulls.  Again, this is not expected in practice, except
        # for degenerate cases checked below.
        l1 = complex(0.0,0.0)
        l2 = complex(0.0, 1.0)
        l3 = complex(0.0,-1.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,2)

    def test_null_classification_a(self):
        # Test for 3-D type A (radial) null
        l1 = complex(-.25,0.0)
        l2 = complex(-.25, 0.0)
        l3 = complex(0.5,0.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,3)

    def test_null_classification_b(self):
        # Test for 3-D type B (radial) null
        l1 = complex(.25,0.0)
        l2 = complex(.25, 0.0)
        l3 = complex(-0.5,0.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,4)


    def test_null_classification_as(self):
        # Test for 3-D type A_s (spiral) null
        l1 = complex(0.5,0.0)
        l2 = complex(-0.25, 0.25)
        l3 = complex(-0.25,-0.25)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,5)


    def test_null_classification_bs(self):
        # Test for 3-D type B_s (spiral) null
        l1 = complex(-0.5,0.0)
        l2 = complex(0.25, 0.25)
        l3 = complex(0.25,-0.25)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,6)


    def test_null_classification_a_degen(self):
        # Test for type A null, degenerating to type X if the ratio of mim to max norm is < 0.25
        l1 = complex(-0.01,0.0)
        l2 = complex(-0.1, 0.0)
        l3 = complex(0.89,0.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,7)


    def test_null_classification_b_degen(self):
        # Test for type B null, degenerating to type X if the ratio of min to max norm is < 0.25
        l1 = complex(0.01,0.0)
        l2 = complex(0.1, 0.0)
        l3 = complex(-0.89,0.0)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,8)


    def test_null_classification_as_degen(self):
        # Test for type A_s null, degenerating to type O if the ratio of max(abs(real)) to min(abs(imag)) is < 0.25
        l1 = complex(0.1, 0.0)
        l2 = complex(-0.1, 0.5)
        l3 = complex(-0.1, -0.5)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,9)


    def test_null_classification_bs_degen(self):
        # Test for type B_s null, degenerating to type O if the ratio of max(abs(real)) to min(abs(imag)) is < 0.25
        l1 = complex(-0.1,0.0)
        l2 = complex(-0.1, 0.5)
        l3 = complex(0.1,-0.5)
        lambdas = [l1,l2,l3]
        tc = pyspedas.classify_null_type(lambdas)
        self.assertEqual(tc,10)


    def test_find_magnetic_nulls_fote_mms(self):
        data = pyspedas.projects.mms.fgm(probe=[1, 2, 3, 4], trange=['2015-09-19/07:40', '2015-09-19/07:45'], data_rate='srvy', time_clip=True, varformat='*_gse_*', get_fgm_ephemeris=True)
        fields = ['mms'+prb+'_fgm_b_gse_srvy_l2' for prb in ['1', '2', '3', '4']]
        positions = ['mms'+prb+'_fgm_r_gse_srvy_l2' for prb in ['1', '2', '3', '4']]
        null_vars = pyspedas.find_magnetic_nulls_fote(fields=fields, positions=positions, smooth_fields=True,smooth_npts=10,smooth_median=True)
        tplot(null_vars,save_png='magnetic_null_vars',display=global_display)
        # We reconstruct the field vectors at the s/c positions using the Jacobian and the field value at the tetrahedron
        # barycenter.  The max difference between the observations and the reconstructed values for each time step
        # should be extremely close to zero, so that's what we'll use for our test.
        d=get_data('max_reconstruction_error')
        np.testing.assert_allclose(d.y,0.0,atol=1.0e-10)

    def test_find_magnetic_nulls_fote_cluster(self):
        data = pyspedas.projects.cluster.load_csa(probes=['C1','C2','C3','C4'],trange=['2003-08-17/16:40', '2003-08-17/16:45'],datatypes='CP_FGM_FULL', time_clip=True)
        #tplot_names()
        fields = ['B_vec_xyz_gse__C'+prb+'_CP_FGM_FULL' for prb in ['1', '2', '3', '4']]
        positions = ['sc_pos_xyz_gse__C'+prb+'_CP_FGM_FULL' for prb in ['1', '2', '3', '4']]
        null_vars = pyspedas.find_magnetic_nulls_fote(fields=fields, positions=positions, smooth_fields=True)
        tplot(null_vars,display=global_display,save_png='cluster_null_vars')
        d=get_data('max_reconstruction_error')
        np.testing.assert_allclose(d.y,0.0,atol=1.0e-10)




if __name__ == '__main__':
    unittest.main()